"use strict";
/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.applySourcemapsToStackTrace = void 0;
const source_map_1 = require("source-map");
// Matches a stack trace line in the form: "at ['async' or empty] [functionName] ([fileName]:[line]:[column])"
// (see https://regex101.com/r/uJJNPd/1). All the tokens are available as named capture groups.
const STACK_TRACE_LINE_REGEX = /at (?<async>async )?(?<functionName>.+) \((?<fileName>.+):(?<line>\d+):(?<column>\d+)\)/m;
/** Applies sourcemaps to a stack trace. */
async function applySourcemapsToStackTrace(scriptName, sourceMap, stackTrace) {
    const sourceMapConsumers = {};
    try {
        const modifiedLines = await Promise.all(stackTrace
            .split('\n')
            .map((line) => applySourcemapsToStackTraceLine(line, scriptName, sourceMap, sourceMapConsumers)));
        return modifiedLines.join('\n');
    }
    catch (e) {
        // Return the original stacktrace if something went wrong
        return stackTrace;
    }
    finally {
        try {
            // Ensure that all source map consumers get destroyed
            for (const consumer of Object.values(sourceMapConsumers)) {
                // The typings of the source-map package are broken
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                consumer.destroy();
            }
        }
        catch (e) {
            // Do nothing
        }
    }
}
exports.applySourcemapsToStackTrace = applySourcemapsToStackTrace;
/** Applies sourcemaps to a single stack trace line */
async function applySourcemapsToStackTraceLine(stackTraceLine, scriptName, sourceMap, sourceMapConsumers) {
    const match = STACK_TRACE_LINE_REGEX.exec(stackTraceLine);
    if (!match || !match.groups) {
        // The stack trace line was not parsable (e.g. because of an anonymous function etc.), so use the original line
        return stackTraceLine;
    }
    // Get the matched tokens
    const { async, functionName, fileName, line, column } = match.groups;
    if (scriptName !== fileName) {
        return stackTraceLine;
    }
    // Get or lazily create the source map consumer
    if (!sourceMapConsumers[fileName]) {
        sourceMapConsumers[fileName] = await new source_map_1.SourceMapConsumer(
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        sourceMap.toString('utf-8'));
    }
    const { source: sourcePath, line: originalLine, column: originalColumn, } = sourceMapConsumers[fileName].originalPositionFor({
        line: Number(line),
        column: Number(column),
    });
    if (sourcePath && originalLine && originalColumn) {
        return `    at ${async ? 'async ' : ''}${functionName} (${sourcePath}:${originalLine}:${originalColumn})`;
    }
    else {
        return stackTraceLine;
    }
}
//# sourceMappingURL=apply-source-maps.js.map