"use strict";
/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.createContext = void 0;
const buffer_1 = require("buffer");
const chalk_1 = require("chalk");
const crypto_1 = require("crypto");
const os_1 = require("os");
const web_1 = require("stream/web");
const node_fetch_1 = require("node-fetch");
const util_1 = require("util");
const v8_1 = require("v8");
const vm = require("vm");
const worker_threads_1 = require("worker_threads");
const atob_1 = require("./polyfills/atob");
const btoa_1 = require("./polyfills/btoa");
const fetch_1 = require("./polyfills/fetch");
const events_1 = require("./polyfills/events");
const domexception_1 = require("./polyfills/domexception");
const location_1 = require("./polyfills/location");
const navigator_1 = require("./polyfills/navigator");
const window_1 = require("./polyfills/window");
const websocket_1 = require("./polyfills/websocket");
/** Creates a new VM context */
function createContext(cwd, urlPathname, config) {
    const environmentUrl = config?.environmentUrl ?? 'http://127.0.0.1';
    const globals = {
        environmentUrl: readOnly(environmentUrl),
    };
    if (config) {
        if (Object.prototype.hasOwnProperty.call(config, 'appId')) {
            globals.appId = readOnly(config.appId);
        }
        if (Object.prototype.hasOwnProperty.call(config, 'appName')) {
            globals.appName = readOnly(config.appName);
        }
        if (Object.prototype.hasOwnProperty.call(config, 'tenantId')) {
            globals.tenantId = readOnly(config.tenantId);
        }
    }
    const href = new URL(environmentUrl);
    href.pathname = urlPathname;
    const location = new location_1.default(href.toString());
    const navigator = Object.create(navigator_1.default);
    Object.defineProperty(navigator, 'hardwareConcurrency', {
        get() {
            return (0, os_1.cpus)().length;
        },
        configurable: true,
        enumerable: true,
    });
    const globalProperties = Object.defineProperties(new window_1.default(), {
        ...globals,
        AbortController: nonEnumerable(AbortController),
        AbortSignal: nonEnumerable(AbortSignal),
        Blob: nonEnumerable(buffer_1.Blob),
        ByteLengthQueuingStrategy: nonEnumerable(web_1.ByteLengthQueuingStrategy),
        CloseEvent: nonEnumerable(websocket_1.CloseEvent),
        CompressionStream: nonEnumerable(undefined),
        CountQueuingStrategy: nonEnumerable(web_1.CountQueuingStrategy),
        CryptoKey: nonEnumerable(crypto_1.webcrypto.CryptoKey),
        CustomEvent: nonEnumerable(events_1.CustomEvent),
        DecompressionStream: nonEnumerable(undefined),
        DOMException: nonEnumerable(domexception_1.default),
        ErrorEvent: nonEnumerable(events_1.ErrorEvent),
        Event: nonEnumerable(Event),
        EventTarget: nonEnumerable(EventTarget),
        File: nonEnumerable(node_fetch_1.File),
        FileReader: nonEnumerable(undefined),
        FormData: nonEnumerable(node_fetch_1.FormData),
        Headers: nonEnumerable(node_fetch_1.Headers),
        Location: {
            value: location_1.default,
            configurable: true,
            writable: true,
        },
        location: {
            get() {
                return location;
            },
            set() {
                // eslint-disable-next-line quotes
                throw new domexception_1.default(`Cannot set "location".`, 'NotSupportedError');
            },
            enumerable: true,
        },
        MessageEvent: nonEnumerable(websocket_1.MessageEvent),
        ProgressEvent: nonEnumerable(events_1.ProgressEvent),
        ReadableStream: nonEnumerable(web_1.ReadableStream),
        ReadableStreamDefaultReader: nonEnumerable(web_1.ReadableStreamDefaultReader),
        TextDecoder: nonEnumerable(util_1.TextDecoder),
        TextEncoder: nonEnumerable(util_1.TextEncoder),
        TextDecoderStream: nonEnumerable(web_1.TextDecoderStream),
        TextEncoderStream: nonEnumerable(web_1.TextEncoderStream),
        TransformStream: nonEnumerable(web_1.TransformStream),
        URL: nonEnumerable(URL),
        URLPattern: nonEnumerable(undefined),
        URLSearchParams: nonEnumerable(URLSearchParams),
        WebSocket: nonEnumerable(websocket_1.WebSocket),
        MessageChannel: nonEnumerable(worker_threads_1.MessageChannel),
        MessagePort: nonEnumerable(worker_threads_1.MessagePort),
        WritableStream: nonEnumerable(web_1.WritableStream),
        WritableStreamDefaultWriter: nonEnumerable(web_1.WritableStreamDefaultWriter),
        WritableStreamDefaultController: nonEnumerable(web_1.WritableStreamDefaultController),
        ReadableByteStreamController: nonEnumerable(web_1.ReadableByteStreamController),
        ReadableStreamBYOBReader: nonEnumerable(web_1.ReadableStreamBYOBReader),
        ReadableStreamBYOBRequest: nonEnumerable(web_1.ReadableStreamBYOBRequest),
        ReadableStreamDefaultController: nonEnumerable(web_1.ReadableStreamDefaultController),
        TransformStreamDefaultController: nonEnumerable(web_1.TransformStreamDefaultController),
        Request: nonEnumerable(node_fetch_1.Request),
        Response: nonEnumerable(node_fetch_1.Response),
        atob: writable(atob_1.default),
        btoa: writable(btoa_1.default),
        setInterval: writable(setInterval),
        setTimeout: writable(setTimeout),
        clearInterval: writable(clearInterval),
        clearTimeout: writable(clearTimeout),
        console: nonEnumerable(new Proxy(console, {
            get(target, prop) {
                switch (prop) {
                    case 'log':
                    case 'info':
                    case 'debug':
                        return (...args) => console[prop].call(console, (0, chalk_1.magenta)(`[${prop.toUpperCase()}]`), (0, chalk_1.yellow)(urlPathname), ...args);
                    case 'warn':
                    case 'error':
                        // Prevent console.warn and console.error from writing to stderr output
                        return (...args) => console.log((0, chalk_1.red)(`[${prop.toUpperCase()}]`), (0, chalk_1.yellow)(urlPathname), ...args);
                    default:
                        return target[prop];
                }
            },
        })),
        crypto: readOnly(crypto_1.webcrypto),
        Crypto: nonEnumerable(crypto_1.webcrypto.constructor),
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        SubtleCrypto: nonEnumerable(crypto_1.webcrypto.subtle),
        fetch: writable((0, fetch_1.default)(cwd, environmentUrl, config)),
        /* eslint-disable @typescript-eslint/no-explicit-any */
        structuredClone: nonEnumerable(globalThis.structuredClone ??
            function (value) {
                return (0, v8_1.deserialize)((0, v8_1.serialize)(value));
            }),
        /* eslint-enable */
        Navigator: nonEnumerable(navigator_1.default),
        navigator: {
            get() {
                return navigator;
            },
            configurable: true,
            enumerable: true,
        },
        Window: {
            value: window_1.default,
            configurable: true,
            enumerable: false,
            writable: true,
        },
    });
    return vm.createContext(Object.setPrototypeOf(Object.defineProperties(globalProperties, {
        window: readOnly(globalProperties),
        self: readOnly(globalProperties),
    }), window_1.default.prototype));
}
exports.createContext = createContext;
/** Writable property descriptor factory */
function writable(
// eslint-disable-next-line @typescript-eslint/no-explicit-any
value) {
    return {
        value,
        configurable: true,
        enumerable: true,
        writable: true,
    };
}
/** Non-enumerable property descriptor factory */
function nonEnumerable(
// eslint-disable-next-line @typescript-eslint/no-explicit-any
value) {
    return {
        value,
        configurable: true,
        enumerable: false,
        writable: true,
    };
}
/** Read-only property descriptor factory */
function readOnly(
// eslint-disable-next-line @typescript-eslint/no-explicit-any
value) {
    return {
        value,
        configurable: true,
        enumerable: true,
        writable: false,
    };
}
//# sourceMappingURL=context.js.map