"use strict";
/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.dispatch = exports.initialize = void 0;
const chalk_1 = require("chalk");
const child_process_1 = require("child_process");
const events_1 = require("events");
const net_1 = require("net");
const path_1 = require("path");
const v8_1 = require("v8");
const readyListener = new events_1.EventEmitter();
let childProcess;
let port = 0;
let state = 0 /* none */;
/** Initialize the serverless worker server */
async function initialize() {
    state = 1 /* initializing */;
    // We must keep the `NODE_OPTIONS` from the environment here so that debugging with VS Code works
    // VS Code passes the following command line flags via `NODE_OPTIONS`:
    // --require "/Applications/Visual Studio Code.app/Contents/Resources/app/extensions/ms-vscode.js-debug/src/bootloader.bundle.js" --enable-source-maps --inspect-publish-uid=http
    const opts = (process.env.NODE_OPTIONS ?? '') +
        ' --experimental-vm-modules --no-warnings --enable-source-maps';
    // Spawn a new child process with the serverless worker server
    childProcess = (0, child_process_1.spawn)(process.execPath, [(0, path_1.join)(__dirname, 'server.js')], {
        env: {
            ...process.env,
            NODE_OPTIONS: opts,
            FORCE_COLOR: '1',
        },
    });
    childProcess.stderr.on('data', (data) => {
        console.error((0, chalk_1.yellow)('[runtime-simulator STDERR]'), data.toString());
    });
    return new Promise((resolve, reject) => {
        const dataListener = (data) => {
            // Wait for the serverless worker server to become ready
            if (data.toString().startsWith('Listening on port')) {
                port = Number.parseInt(data.toString().substring('Listening on port'.length).trim());
                clearTimeout(timeoutHandle);
                childProcess.stdout.removeListener('data', dataListener);
                resolve();
            }
        };
        const timeoutHandle = setTimeout(() => {
            childProcess.stdout.removeListener('data', dataListener);
            reject(new Error('Failed to start the serverless worker server: Timeout'));
        }, 8000);
        childProcess.stdout.on('data', dataListener);
    }).then(() => {
        state = 2 /* ready */;
        childProcess.stdout.on('data', (data) => {
            // Pipe the stdout output to the Console API and remove the last newline character
            console.log(data.toString().replace(/\n$/, ''));
        });
        if (readyListener.listenerCount('') > 0) {
            readyListener.emit('');
        }
    });
}
exports.initialize = initialize;
/**
 * Dispatch a serverless worker task.
 *
 * Creates a new TCP socket connection for every serverless execution to the
 * serverless worker server. Once the response from the server has been read,
 * the worker server will close the socket with FIN.
 */
async function dispatch(task) {
    if (state === 0 /* none */) {
        throw new Error('Failed to dispatch serverless worker task: controller not initialized');
    }
    else if (state === 1 /* initializing */) {
        await new Promise((resolve) => {
            readyListener.once('', () => resolve());
        });
    }
    // Create a new socket
    const socket = new net_1.Socket();
    return new Promise((resolve, reject) => {
        const timeoutHandle = setTimeout(() => {
            reject(new Error('Failed to connect to the serverless worker server: Timeout'));
        }, 8000);
        // Connect to the remote server
        socket.connect(port, 'localhost', () => {
            clearTimeout(timeoutHandle);
            resolve();
        });
    })
        .then(() => {
        return new Promise((resolve, reject) => {
            const chunks = [];
            /** Data event listener */
            function dataListener(chunk) {
                chunks.push(chunk);
            }
            socket.on('data', dataListener);
            socket.once('close', () => {
                socket.removeListener('data', dataListener);
                // Deserialize the RPC response
                const rpcResponse = (0, v8_1.deserialize)(Buffer.concat(chunks));
                if (Object.prototype.hasOwnProperty.call(rpcResponse, 'result')) {
                    resolve(rpcResponse.result);
                }
                else {
                    reject((0, v8_1.deserialize)(rpcResponse.error));
                }
            });
            try {
                const data = (0, v8_1.serialize)(task);
                socket.write(Buffer.concat([
                    // Add the content length
                    Buffer.from(data.byteLength.toString()),
                    data,
                ]), (error) => {
                    if (error) {
                        reject(error);
                    }
                });
            }
            catch (error) {
                reject(error);
            }
        });
    })
        .finally(() => {
        if (socket.readable) {
            socket.end();
        }
    });
}
exports.dispatch = dispatch;
process.once('exit', () => {
    if (childProcess) {
        childProcess.kill();
    }
});
//# sourceMappingURL=controller.js.map