"use strict";
/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const devkit_1 = require("@dynatrace/devkit");
const net_1 = require("net");
const readline = require("readline");
const stream_1 = require("stream");
const v8_1 = require("v8");
const run_1 = require("./run");
const apply_source_maps_1 = require("./apply-source-maps");
const maxConnections = parseInt(process.env.DTP_SERVERLESS_MAX_CONNECTIONS);
const server = (0, net_1.createServer)();
server.maxConnections = isNaN(maxConnections) ? 128 : maxConnections;
server.on('connection', (socket) => {
    const chunks = [];
    let bytesLength = 0;
    /** Data event listener */
    async function dataListener(chunk) {
        if (bytesLength === 0) {
            bytesLength = await new Promise((resolve) => {
                const rl = readline.createInterface({
                    input: new stream_1.PassThrough().end(chunk),
                });
                rl.once('line', (line) => {
                    const dataByteLength = parseInt(line);
                    const selfByteLength = Buffer.from(dataByteLength.toString()).byteLength;
                    // Remove the content length
                    chunks.push(chunk.slice(selfByteLength));
                    resolve(selfByteLength + dataByteLength);
                });
            });
        }
        else {
            chunks.push(chunk);
        }
        // Determine whether all data has been read
        if (socket.bytesRead === bytesLength) {
            socket.emit('end');
        }
    }
    socket.on('data', dataListener);
    socket.once('end', async () => {
        socket.removeListener('data', dataListener);
        // Keep sourceMap and scriptName in case of an error to apply it to the stacktrace.
        let scriptName;
        let sourceMap;
        // Create a new RPC response
        const rpcResult = {};
        try {
            // Deserialize the RPC request
            const rpcRequest = (0, v8_1.deserialize)(Buffer.concat(chunks));
            scriptName = rpcRequest.scriptName;
            sourceMap = rpcRequest.sourceMap;
            rpcResult.result = await (0, run_1.run)(rpcRequest);
        }
        catch (error) {
            if (scriptName && sourceMap && error.stack) {
                // Apply source maps to the stack trace
                error.stack = await (0, apply_source_maps_1.applySourcemapsToStackTrace)(scriptName, sourceMap, error.stack);
            }
            rpcResult.error = (0, v8_1.serialize)(error);
        }
        finally {
            socket.write((0, v8_1.serialize)(rpcResult), (error) => {
                if (error) {
                    console.error(error);
                }
                socket.end();
            });
        }
    });
    socket.on('drain', () => {
        socket.resume();
    });
    socket.on('error', (error) => {
        console.error(error);
    });
    socket.once('close', (error) => {
        if (error) {
            console.error(error);
        }
    });
});
process.once('exit', () => {
    server.close();
});
(0, devkit_1.runOnUnusedPort)((port) => {
    return new Promise((resolve, reject) => {
        const errListener = (error) => {
            reject(error);
        };
        // If error occurs before listening means that server could not be started.
        // That means the promise should be rejected. runOnUnused checks if that error
        // was due to the address being used
        server.once('error', errListener);
        // If listening occurs that means the server was started successfully. Therefore
        // the errListener should be removed immediately. Bind logging error listener instead.
        server.once('listening', () => {
            server.removeListener('error', errListener);
            server.on('error', (error) => {
                console.error(error);
            });
            // Resolve after server was started and error listener is bound.
            resolve();
        });
        server.listen(port, 'localhost');
    });
}, 30000)
    // This is only reached if no port was found or error was different than the address in use error.
    .catch((error) => {
    console.error(error);
})
    // If the server was started successfully, log the used port so the controller can retrieve it.
    .then(() => {
    const address = server.address();
    console.log('Listening on port ' + address.port);
});
//# sourceMappingURL=server.js.map